@@ -1,11 +1,8 @@
 """Module holding the OpenApiReader reader_class implementation."""
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, List, Union
 
 from DataDriver.AbstractReaderClass import AbstractReaderClass
 from DataDriver.ReaderConfig import TestCaseData
-from prance import ResolvingParser
-from prance.util.url import ResolutionError
-from robot.libraries.BuiltIn import BuiltIn
 
 
 # pylint: disable=too-few-public-methods
@@ -33,7 +30,7 @@ class OpenApiReader(AbstractReaderClass):
     def get_data_from_source(self) -> List[TestCaseData]:
         test_data: List[TestCaseData] = []
 
-        endpoints = _get_endpoints_from_source(getattr(self, "source", None))
+        endpoints = getattr(self, "endpoints")
         if ignored_endpoints := getattr(self, "ignored_endpoints", None):
             for endpoint in ignored_endpoints:
                 endpoints.pop(endpoint)
@@ -87,17 +84,6 @@ class OpenApiReader(AbstractReaderClass):
         return test_data
 
 
-def _get_endpoints_from_source(source: Optional[str]) -> Dict[str, Any]:
-    try:
-        parser = ResolvingParser(source, backend="openapi-spec-validator")
-    except (ResolutionError, AssertionError) as exception:
-        BuiltIn().fatal_error(
-            f"Exception while trying to load openapi spec from source: {exception}"
-        )
-    endpoints: Dict[str, Any] = parser.specification["paths"]
-    return endpoints
-
-
 def _get_tag_list(tags: List[str], method: str, response: str) -> List[str]:
     tags.append(f"Method: {method.upper()}")
     tags.append(f"Response: {response}")
