@@ -13,18 +13,21 @@ For more information about the DataDriver library, see
 https://github.com/Snooz82/robotframework-datadriver.
 
 ---
+
 > Note: OpenApiDriver is currently in early development so there are currently
 restrictions / limitations that you may encounter when using this library to run
 tests against an API. See [Limitations](#limitations) for details.
 
 ---
+
 ## Installation
 
 If you already have Python >= 3.8 with pip installed, you can simply run:
 
-``pip install --upgrade robotframework-openapidriver``
+`pip install --upgrade robotframework-openapidriver`
 
 ---
+
 ## OpenAPI (aka Swagger)
 
 The OpenAPI Specification (OAS) defines a standard, language-agnostic interface
@@ -34,68 +37,91 @@ The OpenApiDriver module implements a reader class that generates a test case fo
 each endpoint, method and response that is defined in an OpenAPI document, typically
 an openapi.json or openapi.yaml file.
 
----
-## How it works
+> Note: OpenApiDriver is designed for APIs based on the OAS v3
+The library has not been tested for APIs based on the OAS v2.
 
-If the source file has the .json or .yaml extension, it will be loaded by the
-library (using the prance library under the hood) and the test cases will be generated.
+---
+
+## Getting started
+
+Before trying to use OpenApiDriver to run automatic validations on the target API
+it's recommended to first ensure that the openapi document for the API is valid
+under the OpenAPI Specification.
+
+This can be done using the command line interface of a package that is installed as
+a prerequisite for OpenApiDriver.
+Both a local openapi.json or openapi.yaml file or one hosted by the API server
+can be checked using the `prance validate <reference_to_file>` shell command:
+
+```shell
+prance validate http://localhost:8000/openapi.json
+Processing "http://localhost:8000/openapi.json"...
+ -> Resolving external references.
+Validates OK as OpenAPI 3.0.2!
+
+prance validate /tests/files/petstore_openapi.yaml
+Processing "/tests/files/petstore_openapi.yaml"...
+ -> Resolving external references.
+Validates OK as OpenAPI 3.0.2!
+```
+
+You'll have to change the url or file reference to the location of the openapi
+document for your API.
+
+If the openapi document passes this validation, the next step is trying to do a test
+run with a minimal test suite.
+The example below can be used, with `source` and `origin` altered to fit your situation.
 
 ``` robotframework
 *** Settings ***
 Library            OpenApiDriver
-...                    source=openapi.json
-Test Template      Do Nothing
-
+...                    source=http://localhost:8000/openapi.json
+...                    origin=http://localhost:8000
+Test Template      Validate Using Test Endpoint Keyword
 
 *** Test Cases ***
-Some OpenAPI test for ${method} on ${endpoint} where ${status_code} is expected
+Test Endpoint for ${method} on ${endpoint} where ${status_code} is expected
 
-*** Keywords *** ***
-Do Nothing
+*** Keywords ***
+Validate Using Test Endpoint Keyword
     [Arguments]    ${endpoint}    ${method}    ${status_code}
-    No Operation
+    Test Endpoint
+    ...    endpoint=${endpoint}    method=${method}    status_code=${status_code}
+
 ```
 
-It is also possible to load the openapi.json / openapi.yaml directly from the
-server by using the url instead of a local file:
+Running the above suite for the first time is likely to result in some
+errors / failed testes.
+You should look at the Robot Framework `log.html` to determine the reasons
+for the failing tests.
+Depending on the reasons for the failures, different solutions are possible.
 
-``` robotframework
-*** Settings ***
-Library            OpenApiDriver
-...                    source=http://127.0.0.1:8000/openapi.json
-```
-
-Since the OpenAPI document is essentially a contract that specifies what operations are
-supported and what data needs to be send and will be returned, it is possible to
-automatically validate the API against this contract. For this purpose, the openapi
-module also implements a number of keywords.
-
-Details about the Keywords can be found
+Details about the OpenApiDriver library parameters that you may need can be found
 [here](https://marketsquare.github.io/robotframework-openapidriver/openapidriver.html).
 
 The OpenApiDriver also support handling of relations between resources within the scope
 of the API being validated as well as handling dependencies on resources outside the
 scope of the API. In addition there is support for handling restrictions on the values
 of parameters and properties.
+
 Details about the `mappings_path` variable usage can be found
-[here](https://marketsquare.github.io/robotframework-openapidriver/advanced_use.md).
+[here](https://marketsquare.github.io/robotframework-openapidriver/advanced_use.html).
 
 ---
+
 ## Limitations
 
 There are currently a number of limitations to supported API structures, supported
 data types and properties. The following list details the most important ones:
 - Only JSON request and response bodies are currently supported.
-- The unique identifier for a resource as used in the ``paths`` section of the
-    openapi document is expected to be the ``id`` property on a resource of that type.
-- Limited support for query strings.
-- No support for headers at this time.
+- The unique identifier for a resource as used in the `paths` section of the
+    openapi document is expected to be the `id` property on a resource of that type.
+- Limited support for query strings and headers.
 - Limited support for authentication
-    - ``username`` and ``password`` can be passed as parameters to use Basic Authentication
+    - `username` and `password` can be passed as parameters to use Basic Authentication
     - A [requests AuthBase instance](https://docs.python-requests.org/en/latest/api/#authentication)
         can be passed and it will be used as provided.
     - No support for per-endpoint authorization levels (just simple 401 validation).
-- ``exclusiveMinimum`` and ``exclusiveMaximum`` not supported yet.
 - byte, binary, date, date-time string formats not supported yet.
 
 """
@@ -106,10 +132,13 @@ from typing import List, Optional, Union
 
 from DataDriver import DataDriver
 from DataDriver.AbstractReaderClass import AbstractReaderClass
+from prance import ResolvingParser
+from prance.util.url import ResolutionError
 from requests.auth import AuthBase
-from robotlibcore import DynamicCore
+from robot.api.deco import library
+from robot.libraries.BuiltIn import BuiltIn
 
-from OpenApiDriver.openapi_executors import OpenapiExecutors, ValidationLevel
+from OpenApiDriver.openapi_executors import OpenApiExecutors, ValidationLevel
 from OpenApiDriver.openapi_reader import OpenApiReader
 
 try:
@@ -118,7 +147,8 @@ except:  # pragma: no cover
     __version__ = "unknown"
 
 
-class OpenApiDriver(DataDriver, DynamicCore):
+@library
+class OpenApiDriver(DataDriver, OpenApiExecutors):
     # region: docstring
     """
     Visit the [https://github.com/MarketSquare/robotframework-openapidriver | library page]
@@ -130,6 +160,7 @@ class OpenApiDriver(DataDriver, DynamicCore):
     serves as the ``Test Template``.
 
     The following Keywords are intended to be used in Test Suites:
+
     - ``Test Endpoint``
     - ``Test Invalid Url``
     - ``Test Unauthorized``
@@ -137,8 +168,9 @@ class OpenApiDriver(DataDriver, DynamicCore):
     # endregion
     ROBOT_LIBRARY_VERSION = __version__
     ROBOT_LIBRARY_DOC_FORMAT = "ROBOT"
+    ROBOT_LIBRARY_SCOPE = "TEST SUITE"
 
-    def __init__(  # pylint: disable=too-many-arguments
+    def __init__(  # pylint: disable=too-many-arguments, too-many-locals
         self,
         source: str,
         ignored_endpoints: Optional[List[str]] = None,
@@ -187,7 +219,7 @@ class OpenApiDriver(DataDriver, DynamicCore):
         openapi document. E.g. ``/petshop/v2``.
 
         === mappings_path ===
-
+        See [https://marketsquare.github.io/robotframework-openapidriver/advanced_use.html | here].
 
         === username ===
         The username to be used for Basic Authentication.
@@ -196,7 +228,7 @@ class OpenApiDriver(DataDriver, DynamicCore):
         The password to be used for Basic Authentication.
 
         === security_token ===
-        The token to be used for token based security using the `Authorization` header.
+        The token to be used for token based security using the ``Authorization`` header.
 
         === auth ===
         A [https://docs.python-requests.org/en/latest/api/#authentication | requests AuthBase instance]
@@ -206,6 +238,7 @@ class OpenApiDriver(DataDriver, DynamicCore):
         By default, a ``WARN`` is logged when the Response received after a Request does not
         comply with the schema as defined in the openapi document for the given operation. The
         following values are supported:
+
         - ``DISABLED``: All Response validation errors will be ignored
         - ``INFO``: Any Response validation erros will be logged at ``INFO`` level
         - ``WARN``: Any Response validation erros will be logged at ``WARN`` level
@@ -235,12 +268,23 @@ class OpenApiDriver(DataDriver, DynamicCore):
         ignored_endpoints = ignored_endpoints if ignored_endpoints else []
         ignored_responses = ignored_responses if ignored_responses else []
         ignored_testcases = ignored_testcases if ignored_testcases else []
+        try:
+            parser = ResolvingParser(source, backend="openapi-spec-validator")
+        except ResolutionError as exception:
+            BuiltIn().fatal_error(
+                f"Exception while trying to load openapi spec from source: {exception}"
+            )
+        if (openapi_spec := parser.specification) is None:  # pragma: no cover
+            BuiltIn().fatal_error(
+                "Source was loaded, but no specification was present after parsing."
+            )
+        endpoints = openapi_spec["paths"]
         DataDriver.__init__(
             self,
             # FIXME: Enable when DataDriver accepts AbstractReaderClass subclasses
             # reader_class=OpenApiReader
             reader_class="openapi_reader",
-            source=source,
+            endpoints=endpoints,
             ignored_endpoints=ignored_endpoints,
             ignored_responses=ignored_responses,
             ignored_testcases=ignored_testcases,
@@ -248,8 +292,9 @@ class OpenApiDriver(DataDriver, DynamicCore):
         )
 
         mappings_path = Path(mappings_path).as_posix()
-        openapi_executors = OpenapiExecutors(
-            source=source,
+        OpenApiExecutors.__init__(
+            self,
+            openapi_specification=openapi_spec,
             origin=origin,
             base_path=base_path,
             mappings_path=mappings_path,
@@ -262,7 +307,6 @@ class OpenApiDriver(DataDriver, DynamicCore):
             require_body_for_invalid_url=require_body_for_invalid_url,
             invalid_property_default_response=invalid_property_default_response,
         )
-        DynamicCore.__init__(self, [openapi_executors])
 
     # FIXME: Hack to allow directly loading the OpenApiReader - remove when DataDriver
     # accepts an AbstractReaderClass subclass as reader_class argument
